@@ -3,7 +3,6 @@ package expressions
 import (
 	"fmt"
 
-	"github.com/lmorg/murex/lang/expressions/primitives"
 	"github.com/lmorg/murex/lang/expressions/symbols"
 )
 
@@ -51,7 +50,7 @@ func (tree *ParserT) validateExpression(exec bool) error {
 				return raiseError(tree.expression, node, 0, "expecting a data value")
 			}
 
-			if node.key == symbols.Scalar && exec &&
+			/*if node.key == symbols.Scalar && exec &&
 				(next == nil || next.key > symbols.AssignAndMerge) {
 
 				v, mxDt, err := tree.getVar(scalarNameDetokenised(node.value), varAsValue)
@@ -61,7 +60,7 @@ func (tree *ParserT) validateExpression(exec bool) error {
 				dt := primitives.NewScalar(mxDt, v)
 				node.key = symbols.Calculated
 				node.dt = dt
-			}
+			}*/
 
 			if node.dt != nil {
 				continue
