@@ -6,6 +6,7 @@ import (
 	"regexp"
 	"runtime"
 	"strings"
+	"time"
 
 	"github.com/lmorg/murex/app"
 	"github.com/lmorg/murex/builtins/pipes/term"
@@ -57,6 +58,8 @@ func Start() {
 		}()
 	}
 
+	lang.ShellProcess.StartTime = time.Now()
+
 	// disable this for Darwin (macOS) because the messages it pops up might
 	// spook many macOS users.
 	if runtime.GOOS != "darwin" {
@@ -161,12 +164,13 @@ func ShowPrompt() {
 			prompt = getPrompt()
 			writeTitlebar()
 		}
-		Prompt.SetPrompt(string(prompt))
 
 		if tty.MissingCrLf() {
 			tty.WriteCrLf()
 		}
 
+		Prompt.SetPrompt(string(prompt))
+
 		line, err := Prompt.Readline()
 		if err != nil {
 			switch err {
@@ -256,7 +260,7 @@ func ShowPrompt() {
 			callEvents("after", block)
 
 			fork := lang.ShellProcess.Fork(lang.F_PARENT_VARTABLE | lang.F_NEW_MODULE | lang.F_NO_STDIN)
-			fork.FileRef = &ref.File{Source: &ref.Source{Module: app.ShellModule}}
+			fork.FileRef = ref.NewModule(app.ShellModule)
 			fork.Stderr = term.NewErr(ansi.IsAllowed())
 			fork.PromptId = thisProc
 			fork.CCEvent = lang.ShellProcess.CCEvent
