@@ -2,7 +2,6 @@ package readline
 
 import (
 	"fmt"
-	"os"
 	"strings"
 )
 
@@ -14,8 +13,10 @@ const (
 	previewModeAutocomplete previewModeT = 2
 )
 
+const previewPromptHSpace = 3
+
 func getPreviewWidth(width int) (preview, forward int) {
-	switch {
+	/*switch {
 	case width < 5:
 		return 0, 0
 	case width < 85:
@@ -26,7 +27,8 @@ func getPreviewWidth(width int) (preview, forward int) {
 		preview = width - 4
 	default:
 		preview = 120
-	}
+	}*/
+	preview = width - 3
 
 	forward = width - preview
 	forward -= 2
@@ -46,7 +48,7 @@ type previewCacheT struct {
 	size  *PreviewSizeT
 }
 
-func getPreviewXY() (*PreviewSizeT, error) {
+func (rl *Instance) getPreviewXY() (*PreviewSizeT, error) {
 	width, height, err := GetSize(int(primary.Fd()))
 	if err != nil {
 		return nil, err
@@ -62,7 +64,7 @@ func getPreviewXY() (*PreviewSizeT, error) {
 
 	preview, forward := getPreviewWidth(width)
 	size := &PreviewSizeT{
-		Height:  height / 3,
+		Height:  height - rl.MaxTabCompleterRows - 10, // hintText, multi-line prompts, etc
 		Width:   preview,
 		Forward: forward,
 	}
@@ -70,21 +72,12 @@ func getPreviewXY() (*PreviewSizeT, error) {
 	return size, nil
 }
 
-func (rl *Instance) writePreview() {
-	/*if rl.previewCache != nil {
-		// refresh screen if preview written previously and this one empty
-		defer func() {
-			if rl.previewCache == nil {
-				rl.screenRefresh()
-			}
-		}()
-	}*/
-
+func (rl *Instance) writePreviewStr() string {
 	if rl.previewMode > previewModeClosed && rl.tcr != nil && rl.tcr.Preview != nil {
-		size, err := getPreviewXY()
+		size, err := rl.getPreviewXY()
 		if err != nil || size.Height < 8 || size.Width < 40 {
 			rl.previewCache = nil
-			return
+			return ""
 		}
 
 		item := rl.previewItem
@@ -92,17 +85,14 @@ func (rl *Instance) writePreview() {
 		item = strings.TrimSpace(item)
 
 		lines, pos, err := rl.tcr.Preview(rl.line.Runes(), item, rl.PreviewImages, size)
-		/*if len(lines) == 0 || err != nil {
-			rl.previewCache = nil
-			return
-		}*/
+
 		if err != nil {
 			rl.ForceHintTextUpdate(err.Error())
 		}
-		err = previewDraw(lines[pos:], size)
+		output, err := rl.previewDrawStr(lines[pos:], size)
 		if err != nil {
 			rl.previewCache = nil
-			return
+			return output
 		}
 
 		rl.previewCache = &previewCacheT{
@@ -112,68 +102,60 @@ func (rl *Instance) writePreview() {
 			size:  size,
 		}
 
-		return
+		return output
 	}
 
 	rl.previewCache = nil
+
+	return ""
 }
 
-/*func (rl *Instance) screenRefresh() {
-	if rl.ScreenRefresh == nil {
-		return
-	}
-
-	old := primary
-	primary = os.Stdout
-
-	rl.ScreenRefresh()
-
-	primary = old
-}*/
-
 const (
 	curHome       = "\x1b[H"
 	curPosSave    = "\x1b[s"
 	curPosRestore = "\x1b[u"
 )
 
-func previewDraw(preview []string, size *PreviewSizeT) error {
-	print := func(s string) {
-		_, _ = os.Stdout.WriteString(s)
-	}
+func (rl *Instance) previewDrawStr(preview []string, size *PreviewSizeT) (string, error) {
+	var output string
 
 	pf := fmt.Sprintf("│%%-%ds│\r\n", size.Width)
 
-	print(curPosSave + curHome)
-	defer func() {
-		print(curPosRestore)
-	}()
+	output += curHome
 
-	print(fmt.Sprintf(cursorForwf, size.Forward))
+	output += fmt.Sprintf(cursorForwf, size.Forward)
 	hr := strings.Repeat("─", size.Width)
-	print("╭" + hr + "╮\r\n")
+	output += "╭" + hr + "╮\r\n"
 
 	for i := 0; i <= size.Height; i++ {
-		print(fmt.Sprintf(cursorForwf, size.Forward))
+		output += fmt.Sprintf(cursorForwf, size.Forward)
 
 		if i >= len(preview) {
 			blank := strings.Repeat(" ", size.Width)
-			print("│" + blank + "│\r\n")
+			output += "│" + blank + "│\r\n"
 			continue
 		}
 
-		print(fmt.Sprintf(pf, preview[i]))
+		output += fmt.Sprintf(pf, preview[i])
 	}
 
-	print(fmt.Sprintf(cursorForwf, size.Forward))
-	print("╰" + hr + "╯\r\n")
+	output += fmt.Sprintf(cursorForwf, size.Forward)
+	output += "╰" + hr + "╯\r\n"
 
-	return nil
+	output += rl.previewMoveToPromptStr(size)
+	return output, nil
 }
 
-func (rl *Instance) previewPageUp() {
+func (rl *Instance) previewMoveToPromptStr(size *PreviewSizeT) string {
+	output := curHome
+	output += moveCursorDownStr(size.Height + previewPromptHSpace)
+	output += rl.moveCursorFromStartToLinePosStr()
+	return output
+}
+
+func (rl *Instance) previewPageUpStr() string {
 	if rl.previewCache == nil {
-		return
+		return ""
 	}
 
 	rl.previewCache.pos -= rl.previewCache.len
@@ -181,12 +163,13 @@ func (rl *Instance) previewPageUp() {
 		rl.previewCache.pos = 0
 	}
 
-	_ = previewDraw(rl.previewCache.lines[rl.previewCache.pos:], rl.previewCache.size)
+	output, _ := rl.previewDrawStr(rl.previewCache.lines[rl.previewCache.pos:], rl.previewCache.size)
+	return output
 }
 
-func (rl *Instance) previewPageDown() {
+func (rl *Instance) previewPageDownStr() string {
 	if rl.previewCache == nil {
-		return
+		return ""
 	}
 
 	rl.previewCache.pos += rl.previewCache.len
@@ -197,13 +180,18 @@ func (rl *Instance) previewPageDown() {
 		}
 	}
 
-	_ = previewDraw(rl.previewCache.lines[rl.previewCache.pos:], rl.previewCache.size)
+	output, _ := rl.previewDrawStr(rl.previewCache.lines[rl.previewCache.pos:], rl.previewCache.size)
+	return output
 }
 
-func (rl *Instance) clearPreview() {
+func (rl *Instance) clearPreviewStr() string {
+	var output string
+
 	if rl.previewMode > previewModeClosed {
-		print(seqRestoreBuffer)
-		rl.echo()
+		output = seqRestoreBuffer
+		output += rl.echoStr()
 		rl.previewMode = previewModeClosed
 	}
+
+	return output
 }
