@@ -36,9 +36,10 @@ func delayedSyntaxTimer(rl *Instance, i int32) {
 		return
 	}
 
-	rl.moveCursorToStart()
-	print(sLine)
-	rl.moveCursorFromEndToLinePos()
+	output := rl.moveCursorToStartStr()
+	output += sLine
+	output += rl.moveCursorFromEndToLinePosStr()
+	print(output)
 }
 
 // DelayedTabContext is a custom context interface for async updates to the tab completions
@@ -88,9 +89,11 @@ func (dtc *DelayedTabContext) AppendSuggestions(suggestions []string) {
 	}
 
 	dtc.rl.tabMutex.Unlock()
-	dtc.rl.clearHelpers()
+
+	output := dtc.rl.clearHelpersStr()
 	//dtc.rl.ForceHintTextUpdate(" ")
-	dtc.rl.renderHelpers()
+	output += dtc.rl.renderHelpersStr()
+	print(output)
 }
 
 // AppendDescriptions updates the tab completions with additional suggestions + descriptions asynchronously
@@ -130,7 +133,9 @@ func (dtc *DelayedTabContext) AppendDescriptions(suggestions map[string]string)
 	}
 
 	dtc.rl.tabMutex.Unlock()
-	dtc.rl.clearHelpers()
+
+	output := dtc.rl.clearHelpersStr()
 	//dtc.rl.ForceHintTextUpdate(" ")
-	dtc.rl.renderHelpers()
+	output += dtc.rl.renderHelpersStr()
+	print(output)
 }
